#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "global.h"
#include "functions.h"
#include "util.h"
#include "ranlib.h"

/* 
 Routine checked against mesh2 code to be correct. 
 Produces identical results 
*/
void grow_point(double *initial_price, double *final_price){
  int underlier;
  static double *growth = NULL;

  if(growth == NULL){
     allocate1dDouble(&growth,Dimensions);
  }

  genmn(Parm, growth, Cov1);

  for(underlier=0; underlier<Dimensions; ++underlier){
    final_price[underlier] = exp(growth[underlier]-Covariance[underlier][underlier]/2)*initial_price[underlier];
  }
}

void pathPair(double **path, double **antiPath){
  /* Generates a random path and its antithetic path.
   *
   * When finished, path[time][underlier] contains the path
   * and antiPath[time][underlier] contains the antithetic path.
   */

  int time, underlier;
  double  delta, antiGrowth;
  double *randomNum;

  allocate1dDouble(&randomNum,  Dimensions);  

  for(underlier=0; underlier<Dimensions; underlier++){
    path[0][underlier] = Initial_price;
    antiPath[0][underlier] = Initial_price;
  }
  
  for(time=1; time < Timeslices; time++){
    genmn(Parm, randomNum, Cov1);
    for(underlier=0; underlier<Dimensions; underlier++){
      path[time][underlier] =
        exp(randomNum[underlier]-Covariance[underlier][underlier]/2)*
        path[time-1][underlier];

      delta = randomNum[underlier] - Dividend_adjusted_mean[underlier]; 
      antiGrowth = Dividend_adjusted_mean[underlier] - delta; 

      antiPath[time][underlier] =
        exp(antiGrowth-Covariance[underlier][underlier]/2)*
        antiPath[time-1][underlier];
    }
  }
  deallocate1dDouble(&randomNum,  Dimensions); 
}
